/***********************************************************************************************************************
 *
 * SolidBlue - open source safe data
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://solidblue.java.net
 * SCM: https://bitbucket.org/tidalwave/solidblue-src
 *
 **********************************************************************************************************************/
package it.tidalwave.integritychecker.impl;

import it.tidalwave.integritychecker.impl.FingerprintComputer;
import javax.annotation.Nonnull;
import java.util.Random;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import it.tidalwave.integritychecker.Fingerprint;
import lombok.Cleanup;
import lombok.extern.slf4j.Slf4j;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.MatcherAssert.*;

/***********************************************************************************************************************
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@Slf4j
public class FingerprintComputerTest 
  {
    private final static int MIN_FILE_SIZE =  1 * 1000 * 1000;
    private final static int MAX_FILE_SIZE = 10 * 1000 * 1000;
    
    private FingerprintComputer fixture;
    
    private File testFileFolder;
    
    @BeforeClass
    private void createTestFiles() 
      throws IOException
      {
        log.info("Creating test files...");
        testFileFolder = new File("target/test-files");
        testFileFolder.mkdirs();
        final Random random = new Random(342345426536L);
        
        for (int f = 0; f < 20; f++)
          {
            final File file = new File(testFileFolder, "testfile" + f);
            log.info(">>>> creating {} ...", file.getAbsolutePath());
            int size = MIN_FILE_SIZE + random.nextInt(MAX_FILE_SIZE - MIN_FILE_SIZE);
            final byte[] buffer = new byte[size];
            random.nextBytes(buffer);
            final @Cleanup FileOutputStream fos = new FileOutputStream(file);
            fos.write(buffer);
            fos.close();
          }
      }
    
    @BeforeMethod
    public void setupFixture()
      {
        fixture = new FingerprintComputer();
      }
    
    @Test(dataProvider="checksums")
    public void test (final @Nonnull String fileName, final @Nonnull String expectedChecksum)
      throws Exception
      { 
          log.info("{} {}", fileName, expectedChecksum);
        final Fingerprint fingerprint = fixture.computeFingerprint(new File(testFileFolder, fileName), "MD5");
        assertThat(fingerprint.getType(), is("MD5"));
        assertThat(fingerprint.getValue(), is(expectedChecksum));
      }
    
    @DataProvider(name="checksums")
    private Object[][] df()
      {
        return new Object[][]
          {
            new Object[] { "testfile0",  "b1b380eae2dcc1f1b07f7aa6412e21f4" },
            new Object[] { "testfile1",  "fec1408f84ff9cd242962c2005b2fbc6" },
            new Object[] { "testfile2",  "9100d0bc7ad32c485ab0c42a28cb7d0a" },
            new Object[] { "testfile3",  "cbf54b90f1fcc0336b3f64b071b396f6" },
            new Object[] { "testfile4",  "eab5707051d0f450c0e5e446c9aa85bb" },
            new Object[] { "testfile5",  "c03186b388cd30aa4fd30f2e964157e3" },
            new Object[] { "testfile6",  "d17fd747746f5b0f1a6582c89ccf2ae0" },
            new Object[] { "testfile7",  "026108ca90472697193144964c1c0b51" },
            new Object[] { "testfile8",  "1eb0e9c6afe921d034262371bace7caf" },
            new Object[] { "testfile9",  "4344077c302c9110cda4ab80a01db6fb" },               
            new Object[] { "testfile10", "dced7a94d99268c15777b6c2a4487766" },
            new Object[] { "testfile11", "e91c87bb8ff03bd22b27c6efd4dc983d" },
            new Object[] { "testfile12", "7ea6a6b6859e3eeb7ca1ffb19f4a3d37" },
            new Object[] { "testfile13", "9d36d793766376f3175779b5c6e5b647" },
            new Object[] { "testfile14", "075e1824557262bfbea98016bf83ab6a" },
            new Object[] { "testfile15", "207841cd71d0d95b11d3e1463cfad3f8" },
            new Object[] { "testfile16", "f1f968748849733a00a00ced4b8858a5" },
            new Object[] { "testfile17", "78dde0180bef08dd3a6f8a7efbf6f71e" },
            new Object[] { "testfile18", "1c84679f6c6d11c9f56201eceb8a4f30" },
            new Object[] { "testfile19", "64ec50d90aee2cf897950e750e3a6425" }
          };
      }
  }
